unit ListViewEx;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ShellApi;

type
  TFilesDropped = procedure(DroppedFiles: TStringList) of Object;
  TExplorerListView = class(TListView)
  private

    //Define Event
    FFilesDropped: TFilesDropped;

    //Proc Pointers
    OldWndProc: TFarProc;
    NewWndProc: Pointer;

    //Field Variables
    FList  : TStringList; // Store dropped files
    FBTF   : Boolean; // Bring to front?

    ParentHandle: THandle;

    procedure HookParent;
    procedure UnhookParent;
    procedure HookWndProc(var Message: TMessage);

  protected
    { Protected declarations }
  public
    { Public declarations }

    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    property    DroppedFiles: TStringList Read FList;
    procedure   Activate(YesNo: Boolean);

  published
    { Published declarations }
    procedure ClearList;
    property MoveToFront   : Boolean Read FBTF    Write FBTF;
    property OnFilesDropped: TFilesDropped Read FFilesDropped Write FFilesDropped;
  end;

procedure Register;

implementation

constructor TExplorerListView.Create(AOwner:TComponent);
Begin
  Inherited Create(AOwner);

  NewWndProc := Nil;
  OldWndProc := Nil;

  FList := TStringList.Create;
  FBTF  := False;

End;

destructor TExplorerListView.Destroy;
begin
  UnhookParent;
  FList.Free;

  inherited Destroy;  // Call TComponent's default destroy meathod
end;

procedure TExplorerListView.HookParent;
begin
  // Exit if parent doesn't exist
  if ((Owner As TWinControl) = NIL) And
    (NewWndProc <> nil) then exit;

  //Install Hook
  ParentHandle := Self.Handle;
  OldWndProc := TFarProc(GetWindowLong(ParentHandle, GWL_WNDPROC));
  NewWndProc := MakeObjectInstance(HookWndProc);
  SetWindowLong(ParentHandle, GWL_WNDPROC, LongInt(NewWndProc));
end;

procedure TExplorerListView.UnhookParent;
begin
  if ((Owner As TWinControl) <> NIL) And (Assigned(OldWndProc)) then
    SetWindowLong(ParentHandle, GWL_WNDPROC, LongInt(OldWndProc));
  if assigned(NewWndProc) then
    FreeObjectInstance(NewWndProc);
  NewWndProc := NIL;
  OldWndProc := NIL;
end;

procedure TExplorerListView.HookWndProc(var Message: TMessage);
Var
  I, nl, nFiles:Integer;
  fName: Array[1..255] of Char;
begin
  if (Owner As TWinControl) = NIL then Exit;

  With Message Do Begin
    Result := CallWindowProc(OldWndProc, ParentHandle, Msg, wParam, lParam);

    If (Msg = WM_DROPFILES) Then Begin
      If FBTF Then
        SetForegroundWindow(ParentHandle);

     //Clear the internal list
     FList.Clear;

     //Using -1 in the index gives me the number of files
     nFiles := DragQueryFile(wParam, -1, Nil, 0);
     For i:=0 to nFiles-1 do Begin
       //DragQueryFile returns the length of the given file
       nl := DragQueryFile(wParam,i,@fName,255)+1;

       //Append a character zero to make null-terminated string
       fName[nl] := #0;

       //Add the item
       FList.Add(fName);
     End;

     if ((nFiles > 0) And (Assigned(FFilesDropped))) Then
       FFilesDropped(FList);

    end;
  end;
end;

procedure TExplorerListView.ClearList;
Begin
  Try
    FList.Clear;
  Except
  End;
End;

procedure TExplorerListView.Activate(YesNo: Boolean);
Begin

  HookParent;

  If YesNo Then
    DragAcceptFiles(ParentHandle, True)
  Else
    DragAcceptFiles(ParentHandle, False);
End;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure Register;
begin
  RegisterComponents('Custom', [TExplorerListView]);
end;

end.
